@@ -17,11 +17,10 @@ import zipfile
 import logging
 from struct import unpack
 import hashlib
-import binascii
+import warnings
 
 import lxml.sax
 from xml.dom.pulldom import SAX2DOM
-from enum import Enum
 
 # Used for reading Certificates
 from asn1crypto import cms, x509, keys
@@ -217,13 +216,20 @@ class APK(object):
         0x0301 : "DSA with SHA2-256 digest",
     }
 
+    __no_magic = False
+
     def __init__(self, filename, raw=False, magic_file=None, skip_analysis=False, testzip=False):
         """
         This class can access to all elements in an APK file
 
+        example::
+
+            APK("myfile.apk")
+            APK(read("myfile.apk"), raw=True)
+
         :param filename: specify the path of the file, or raw data
         :param raw: specify if the filename is a path or raw data (optional)
-        :param magic_file: specify the magic file (optional)
+        :param magic_file: specify the magic file (not used anymore - legacy only)
         :param skip_analysis: Skip the analysis, e.g. no manifest files are read. (default: False)
         :param testzip: Test the APK for integrity, e.g. if the ZIP file is broken. Throw an exception on failure (default False)
 
@@ -233,10 +239,10 @@ class APK(object):
         :type skip_analysis: boolean
         :type testzip: boolean
 
-        :Example:
-            APK("myfile.apk")
-            APK(read("myfile.apk"), raw=True)
         """
+        if magic_file:
+            log.warning("You set magic_file but this parameter is actually unused. You should remove it.")
+
         self.filename = filename
 
         self.xml = {}
@@ -249,7 +255,7 @@ class APK(object):
         self.uses_permissions = []
         self.declared_permissions = {}
         self.valid_apk = False
-        
+
         self._is_signed_v2 = None
         self._is_signed_v3 = None
         self._v2_blocks = {}
@@ -259,8 +265,6 @@ class APK(object):
         self._files = {}
         self.files_crc32 = {}
 
-        self.magic_file = magic_file
-
         if raw is True:
             self.__raw = bytearray(filename)
             self._sha256 = hashlib.sha256(self.__raw).hexdigest()
@@ -289,9 +293,10 @@ class APK(object):
         if not skip_analysis:
             self._apk_analysis()
 
-    def _ns(self, name):
+    @staticmethod
+    def _ns(name):
         """
-        return the name including the Android namespace
+        return the name including the Android namespace URI
         """
         return NS_ANDROID + name
 
@@ -326,29 +331,30 @@ class APK(object):
                     log.error("AndroidManifest.xml does not start with a <manifest> tag! Is this a valid APK?")
                     return
 
-                self.package = self.xml[i].get("package")
-                self.androidversion["Code"] = self.xml[i].get(self._ns("versionCode"))
-                self.androidversion["Name"] = self.xml[i].get(self._ns("versionName"))
+                self.package = self.get_attribute_value("manifest", "package")
+                self.androidversion["Code"] = self.get_attribute_value("manifest", "versionCode")
+                self.androidversion["Name"] = self.get_attribute_value("manifest", "versionName")
+                permission = list(self.get_all_attribute_value("uses-permission", "name"))
+                self.permissions = list(set(self.permissions + permission))
 
-                for item in self.xml[i].findall('uses-permission'):
-                    name = item.get(self._ns("name"))
-                    self.permissions.append(name)
-                    maxSdkVersion = None
-                    try:
-                        maxSdkVersion = int(item.get(self._ns('maxSdkVersion')))
-                    except ValueError:
-                        log.warning(item.get(self._ns('maxSdkVersion')) + 'is not a valid value for <uses-permission> maxSdkVersion')
-                    except TypeError:
-                        pass
-                    self.uses_permissions.append([name, maxSdkVersion])
+                for uses_permission in self.find_tags("uses-permission"):
+                    self.uses_permissions.append([
+                        self.get_value_from_tag(uses_permission, "name"),
+                        self._get_permission_maxsdk(uses_permission)
+                    ])
 
                 # getting details of the declared permissions
-                for d_perm_item in self.xml[i].findall('permission'):
-                    d_perm_name = self._get_res_string_value(str(d_perm_item.get(self._ns("name"))))
-                    d_perm_label = self._get_res_string_value(str(d_perm_item.get(self._ns("label"))))
-                    d_perm_description = self._get_res_string_value(str(d_perm_item.get(self._ns("description"))))
-                    d_perm_permissionGroup = self._get_res_string_value(str(d_perm_item.get(self._ns("permissionGroup"))))
-                    d_perm_protectionLevel = self._get_res_string_value(str(d_perm_item.get(self._ns("protectionLevel"))))
+                for d_perm_item in self.find_tags('permission'):
+                    d_perm_name = self._get_res_string_value(
+                        str(self.get_value_from_tag(d_perm_item, "name")))
+                    d_perm_label = self._get_res_string_value(
+                        str(self.get_value_from_tag(d_perm_item, "label")))
+                    d_perm_description = self._get_res_string_value(
+                        str(self.get_value_from_tag(d_perm_item, "description")))
+                    d_perm_permissionGroup = self._get_res_string_value(
+                        str(self.get_value_from_tag(d_perm_item, "permissionGroup")))
+                    d_perm_protectionLevel = self._get_res_string_value(
+                        str(self.get_value_from_tag(d_perm_item, "protectionLevel")))
 
                     d_perm_details = {
                         "label": d_perm_label,
@@ -370,7 +376,7 @@ class APK(object):
         We remove the zip from the Object, as it is not pickable
         And it does not make any sense to pickle it anyways.
 
-        :return: the picklable APK Object without zip.
+        :returns: the picklable APK Object without zip.
         """
         # Upon pickling, we need to remove the ZipFile
         x = self.__dict__
@@ -408,6 +414,16 @@ class APK(object):
                 break
         return string_value
 
+    def _get_permission_maxsdk(self, item):
+        maxSdkVersion = None
+        try:
+            maxSdkVersion = int(self.get_value_from_tag(item, "maxSdkVersion"))
+        except ValueError:
+            log.warning(self.get_max_sdk_version() + 'is not a valid value for <uses-permission> maxSdkVersion')
+        except TypeError:
+            pass
+        return maxSdkVersion
+
     def is_valid_APK(self):
         """
         Return true if the APK is valid, false otherwise.
@@ -440,10 +456,16 @@ class APK(object):
         :rtype: :class:`str`
         """
 
-        app_name = self.get_element('application', 'label')
-        if not app_name:
-            main_activity_name = self.get_main_activity()
-            app_name = self.get_element('activity', 'label', name=main_activity_name)
+        app_name = self.get_attribute_value('application', 'label')
+        if app_name is None:
+            activities = self.get_main_activities()
+            main_activity_name = None
+            if len(activities) > 0:
+                main_activity_name = activities.pop()
+
+            # FIXME: would need to use _format_value inside get_attribute_value for each returned name!
+            # For example, as the activity name might be foobar.foo.bar but inside the activity it is only .bar
+            app_name = self.get_attribute_value('activity', 'label', name=main_activity_name)
 
         if app_name is None:
             # No App name set
@@ -512,10 +534,11 @@ class APK(object):
         For example adaptive icons are usually marked as anydpi.
 
         When it comes now to selecting an icon, there is the following flow:
-        1) is there an anydpi icon?
-        2) is there an icon for the dpi of the device?
-        3) is there a nodpi icon?
-        4) (only on very old devices) is there a icon with dpi 0 (the default)
+
+        1. is there an anydpi icon?
+        2. is there an icon for the dpi of the device?
+        3. is there a nodpi icon?
+        4. (only on very old devices) is there a icon with dpi 0 (the default)
 
         For more information read here: https://stackoverflow.com/a/34370735/446140
 
@@ -523,10 +546,11 @@ class APK(object):
         """
         main_activity_name = self.get_main_activity()
 
-        app_icon = self.get_element('activity', 'icon', name=main_activity_name)
+        app_icon = self.get_attribute_value(
+            'activity', 'icon', name=main_activity_name)
 
         if not app_icon:
-            app_icon = self.get_element('application', 'icon')
+            app_icon = self.get_attribute_value('application', 'icon')
 
         res_parser = self.get_android_resources()
         if not res_parser:
@@ -607,48 +631,45 @@ class APK(object):
         """
         Return the filetype guessed for a buffer
         :param buffer: bytes
-        :return: str of filetype
+        :returns: str of filetype
         """
-        # TODO this functions should be better in another package
         default = "Unknown"
-        ftype = None
 
-        # There are several implementations of magic,
-        # unfortunately all called magic
-        try:
-            import magic
-        except ImportError:
-            # no lib magic at all, return unknown
+        # Faster way, test once, return default.
+        if self.__no_magic:
             return default
 
         try:
-            # We test for the python-magic package here
+            # Magic is optional
+            import magic
+        except ImportError:
+            self.__no_magic = True
+            log.warning("No Magic library was found on your system.")
+            return default
+        except TypeError as e:
+            self.__no_magic = True
+            log.warning("It looks like you have the magic python package installed but not the magic library itself!")
+            log.warning("Error from magic library: %s", e)
+            log.warning("Please follow the installation instructions at https://github.com/ahupp/python-magic/#installation")
+            return default
+
+        try:
+            # There are several implementations of magic,
+            # unfortunately all called magic
+            # We use this one: https://github.com/ahupp/python-magic/
             getattr(magic, "MagicException")
         except AttributeError:
-            try:
-                # Check for filemagic package
-                getattr(magic.Magic, "id_buffer")
-            except AttributeError:
-                # Here, we load the file-magic package
-                ms = magic.open(magic.MAGIC_NONE)
-                ms.load()
-                ftype = ms.buffer(buffer[:1024 * 1024])
-            else:
-                # This is now the filemagic package
-                if self.magic_file is not None:
-                    m = magic.Magic(paths=[self.magic_file])
-                else:
-                    m = magic.Magic()
-                ftype = m.id_buffer(buffer[:1024 * 1024])
-        else:
-            # This is the code for python-magic
-            if self.magic_file is not None:
-                m = magic.Magic(magic_file=self.magic_file)
-            else:
-                m = magic.Magic()
-            ftype = m.from_buffer(buffer[:1024 * 1024])
+            self.__no_magic = True
+            log.warning("Not the correct Magic library was found on your system. Please install python-magic!")
+            return default
 
-        if ftype is None:
+        try:
+            ftype = magic.from_buffer(buffer[:1024])
+        except magic.MagicError as e:
+            log.exception("Error getting the magic type: %s", e)
+            return default
+
+        if not ftype:
             return default
         else:
             return self._patch_magic(buffer, ftype)
@@ -658,7 +679,7 @@ class APK(object):
         """
         Returns a dictionary of filenames and detected magic type
 
-        :return: dictionary of files and their mime type
+        :returns: dictionary of files and their mime type
         """
         return self.get_files_types()
 
@@ -666,15 +687,14 @@ class APK(object):
         """
         Return the files inside the APK with their associated types (by using python-magic)
 
+        At the same time, the CRC32 are calculated for the files.
+
         :rtype: a dictionnary
         """
         if self._files == {}:
             # Generate File Types / CRC List
             for i in self.get_files():
-                buffer = self.zip.read(i)
-                self.files_crc32[i] = crc32(buffer)
-                # FIXME why not use the crc from the zipfile?
-                #crc = self.zip.getinfo(i).CRC
+                buffer = self._get_crc32(i)
                 self._files[i] = self._get_file_magic_name(buffer)
 
         return self._files
@@ -685,27 +705,41 @@ class APK(object):
 
         :param buffer: bytes of the file to detect
         :param orig: guess by mime libary
-        :return: corrected guess
+        :returns: corrected guess
         """
-        if ("Zip" in orig) or ("DBase" in orig):
-            val = androconf.is_android_raw(buffer)
-            if val == "APK":
-                return "Android application package file"
-            elif val == "AXML":
-                return "Android's binary XML"
+        if ("Zip" in orig) or ('(JAR)' in orig) and androconf.is_android_raw(buffer) == 'APK':
+            return "Android application package file"
 
         return orig
 
+    def _get_crc32(self, filename):
+        """
+        Calculates and compares the CRC32 and returns the raw buffer.
+
+        The CRC32 is added to `files_crc32` dictionary, if not present.
+
+        :param filename: filename inside the zipfile
+        :rtype: bytes
+        """
+        buffer = self.zip.read(filename)
+        if filename not in self.files_crc32:
+            self.files_crc32[filename] = crc32(buffer)
+            if self.files_crc32[filename] != self.zip.getinfo(filename).CRC:
+                log.error("File '{}' has different CRC32 after unpacking! "
+                          "Declared: {:08x}, Calculated: {:08x}".format(filename,
+                                                                        self.zip.getinfo(filename).CRC,
+                                                                        self.files_crc32[filename]))
+        return buffer
+
     def get_files_crc32(self):
         """
         Calculates and returns a dictionary of filenames and CRC32
 
-        :return: dict of filename: CRC32
+        :returns: dict of filename: CRC32
         """
         if self.files_crc32 == {}:
             for i in self.get_files():
-                buffer = self.zip.read(i)
-                self.files_crc32[i] = crc32(buffer)
+                self._get_crc32(i)
 
         return self.files_crc32
 
@@ -750,7 +784,8 @@ class APK(object):
         try:
             return self.get_file("classes.dex")
         except FileNotPresent:
-            return ""
+            # TODO is this a good idea to return an empty string?
+            return b""
 
     def get_dex_names(self):
         """
@@ -776,18 +811,22 @@ class APK(object):
         """
         Test if the APK has multiple DEX files
 
-        :return: True if multiple dex found, otherwise False
+        :returns: True if multiple dex found, otherwise False
         """
         dexre = re.compile("^classes(\d+)?.dex$")
         return len([instance for instance in self.get_files() if dexre.search(instance)]) > 1
 
     def get_elements(self, tag_name, attribute, with_namespace=True):
         """
+        .. deprecated:: 3.3.5
+            use :meth:`get_all_attribute_value` instead
+
         Return elements in xml files which match with the tag name and the specific attribute
 
-        :param tag_name: a string which specify the tag name
-        :param attribute: a string which specify the attribute
+        :param str tag_name: a string which specify the tag name
+        :param str attribute: a string which specify the attribute
         """
+        warnings.warn("This method is deprecated since 3.3.5.", DeprecationWarning)
         for i in self.xml:
             if self.xml[i] is None:
                 continue
@@ -802,33 +841,35 @@ class APK(object):
 
     def _format_value(self, value):
         """
-        Format a value with packagename, if not already set
+        Format a value with packagename, if not already set.
+        For example, the name :code:`'.foobar'` will be transformed into :code:`'package.name.foobar'`.
+
+        Names which do not contain any dots are assumed to be packagename-less as well:
+        :code:`foobar` will also transform into :code:`package.name.foobar`.
 
         :param value:
-        :return:
+        :returns:
         """
         if len(value) > 0:
-            if value[0] == ".":
-                value = self.package + value
-            else:
-                v_dot = value.find(".")
-                if v_dot == 0:
-                    value = self.package + "." + value
-                elif v_dot == -1:
-                    value = self.package + "." + value
+            v_dot = value.find(".")
+            if v_dot == 0:
+                value = self.package + "." + value
+            elif v_dot == -1:
+                value = self.package + "." + value
         return value
 
     def get_element(self, tag_name, attribute, **attribute_filter):
         """
+        .. deprecated:: 3.3.5
+            use :meth:`get_attribute_value` instead
+
         Return element in xml files which match with the tag name and the specific attribute
 
-        :param tag_name: specify the tag name
-        :type tag_name: string
-        :param attribute: specify the attribute
-        :type attribute: string
-
-        :rtype: string
+        :param str tag_name: specify the tag name
+        :param str attribute: specify the attribute
+        :rtype: str
         """
+        warnings.warn("This method is deprecated since 3.3.5.", DeprecationWarning)
         for i in self.xml:
             if self.xml[i] is None:
                 continue
@@ -852,6 +893,151 @@ class APK(object):
                     return value
         return None
 
+    def get_all_attribute_value(
+        self, tag_name, attribute, format_value=True, **attribute_filter
+    ):
+        """
+        Yields all the attribute values in xml files which match with the tag name and the specific attribute
+
+        :param str tag_name: specify the tag name
+        :param str attribute: specify the attribute
+        :param bool format_value: specify if the value needs to be formatted with packagename
+        """
+        tags = self.find_tags(tag_name, **attribute_filter)
+        for tag in tags:
+            value = tag.get(attribute) or tag.get(self._ns(attribute))
+            if value is not None:
+                if format_value:
+                    yield self._format_value(value)
+                else:
+                    yield value
+
+    def get_attribute_value(
+        self, tag_name, attribute, format_value=False, **attribute_filter
+    ):
+        """
+        Return the attribute value in xml files which matches the tag name and the specific attribute
+
+        :param str tag_name: specify the tag name
+        :param str attribute: specify the attribute
+        :param bool format_value: specify if the value needs to be formatted with packagename
+        """
+
+        for value in self.get_all_attribute_value(
+                tag_name, attribute, format_value, **attribute_filter):
+            if value is not None:
+                return value
+
+    def get_value_from_tag(self, tag, attribute):
+        """
+        Return the value of the android prefixed attribute in a specific tag.
+
+        This function will always try to get the attribute with a android: prefix first,
+        and will try to return the attribute without the prefix, if the attribute could not be found.
+        This is useful for some broken AndroidManifest.xml, where no android namespace is set,
+        but could also indicate malicious activity (i.e. wrongly repackaged files).
+        A warning is printed if the attribute is found without a namespace prefix.
+
+        If you require to get the exact result you need to query the tag directly:
+
+        example::
+            >>> from lxml.etree import Element
+            >>> tag = Element('bar', nsmap={'android': 'http://schemas.android.com/apk/res/android'})
+            >>> tag.set('{http://schemas.android.com/apk/res/android}foobar', 'barfoo')
+            >>> tag.set('name', 'baz')
+            # Assume that `a` is some APK object
+            >>> a.get_value_from_tag(tag, 'name')
+            'baz'
+            >>> tag.get('name')
+            'baz'
+            >>> tag.get('foobar')
+            None
+            >>> a.get_value_from_tag(tag, 'foobar')
+            'barfoo'
+
+        :param lxml.etree.Element tag: specify the tag element
+        :param str attribute: specify the attribute name
+        :returns: the attribute's value, or None if the attribute is not present
+        """
+
+        # TODO: figure out if both android:name and name tag exist which one to give preference:
+        # currently we give preference for the namespace one and fallback to the un-namespaced
+        value = tag.get(self._ns(attribute))
+        if value is None:
+            value = tag.get(attribute)
+
+            if value:
+                # If value is still None, the attribute could not be found, thus is not present
+                log.warning("Failed to get the attribute '{}' on tag '{}' with namespace. "
+                            "But found the same attribute without namespace!".format(attribute, tag.tag))
+        return value
+
+    def find_tags(self, tag_name, **attribute_filter):
+        """
+        Return a list of all the matched tags in all available xml
+
+        :param str tag: specify the tag name
+        """
+        all_tags = [
+            self.find_tags_from_xml(
+                i, tag_name, **attribute_filter
+            )
+            for i in self.xml
+        ]
+        return [tag for tag_list in all_tags for tag in tag_list]
+
+    def find_tags_from_xml(
+        self, xml_name, tag_name, **attribute_filter
+    ):
+        """
+        Return a list of all the matched tags in a specific xml
+        w
+        :param str xml_name: specify from which xml to pick the tag from
+        :param str tag_name: specify the tag name
+        """
+        xml = self.xml[xml_name]
+        if xml is None:
+            return []
+        if xml.tag == tag_name:
+            if self.is_tag_matched(
+                xml.tag, **attribute_filter
+            ):
+                return [xml]
+            return []
+        tags = xml.findall(".//" + tag_name)
+        return [
+            tag for tag in tags if self.is_tag_matched(
+                tag, **attribute_filter
+            )
+        ]
+
+    def is_tag_matched(self, tag, **attribute_filter):
+        """
+        Return true if the attributes matches in attribute filter.
+
+        An attribute filter is a dictionary containing: {attribute_name: value}.
+        This function will return True if and only if all attributes have the same value.
+        This function allows to set the dictionary via kwargs, thus you can filter like this:
+
+        example::
+            a.is_tag_matched(tag, name="foobar", other="barfoo")
+
+        This function uses a fallback for attribute searching. It will by default use
+        the namespace variant but fall back to the non-namespace variant.
+        Thus specifiying :code:`{"name": "foobar"}` will match on :code:`<bla name="foobar" \>`
+        as well as on :code:`<bla android:name="foobar" \>`.
+
+        :param lxml.etree.Element tag: specify the tag element
+        :param attribute_filter: specify the attribute filter as dictionary
+        """
+        if len(attribute_filter) <= 0:
+            return True
+        for attr, value in attribute_filter.items():
+            _value = self.get_value_from_tag(tag, attr)
+            if _value != value:
+                return False
+        return True
+
     def get_main_activities(self):
         """
         Return names of the main activities
@@ -915,7 +1101,7 @@ class APK(object):
 
         :rtype: a list of str
         """
-        return list(self.get_elements("activity", "name"))
+        return list(self.get_all_attribute_value("activity", "name"))
 
     def get_services(self):
         """
@@ -923,7 +1109,7 @@ class APK(object):
 
         :rtype: a list of str
         """
-        return list(self.get_elements("service", "name"))
+        return list(self.get_all_attribute_value("service", "name"))
 
     def get_receivers(self):
         """
@@ -931,7 +1117,7 @@ class APK(object):
 
         :rtype: a list of string
         """
-        return list(self.get_elements("receiver", "name"))
+        return list(self.get_all_attribute_value("receiver", "name"))
 
     def get_providers(self):
         """
@@ -939,7 +1125,7 @@ class APK(object):
 
         :rtype: a list of string
         """
-        return list(self.get_elements("provider", "name"))
+        return list(self.get_all_attribute_value("provider", "name"))
 
     def get_intent_filters(self, itemtype, name):
         """
@@ -951,7 +1137,7 @@ class APK(object):
 
         :param itemtype: the type of parent item to look for, e.g. `activity`,  `service` or `receiver`
         :param name: the `android:name` of the parent item, e.g. activity name
-        :return: a dictionary with the keys `action` and `category` containing the `android:name` of those items
+        :returns: a dictionary with the keys `action` and `category` containing the `android:name` of those items
         """
         d = {"action": [], "category": []}
 
@@ -1039,7 +1225,10 @@ class APK(object):
 
     def get_details_permissions(self):
         """
-        Return permissions with details
+        Return permissions with details.
+
+        THis can only return details about the permission, if the permission is
+        defined in the AOSP.
 
         :rtype: dict of {permission: [protectionLevel, label, description]}
         """
@@ -1055,15 +1244,18 @@ class APK(object):
                         "Unknown permission from android reference"]
         return l
 
-    @DeprecationWarning
     def get_requested_permissions(self):
         """
+        .. deprecated:: 3.1.0
+            use :meth:`get_permissions` instead.
+
         Returns all requested permissions.
 
         It has the same result as :meth:`get_permissions` and might be removed in the future
 
         :rtype: list of str
         """
+        warnings.warn("This method is deprecated since 3.1.0.", DeprecationWarning)
         return self.get_permissions()
 
     def get_requested_aosp_permissions(self):
@@ -1131,7 +1323,7 @@ class APK(object):
 
             :rtype: string
         """
-        return self.get_element("uses-sdk", "maxSdkVersion")
+        return self.get_attribute_value("uses-sdk", "maxSdkVersion")
 
     def get_min_sdk_version(self):
         """
@@ -1139,7 +1331,7 @@ class APK(object):
 
             :rtype: string
         """
-        return self.get_element("uses-sdk", "minSdkVersion")
+        return self.get_attribute_value("uses-sdk", "minSdkVersion")
 
     def get_target_sdk_version(self):
         """
@@ -1147,7 +1339,7 @@ class APK(object):
 
             :rtype: string
         """
-        return self.get_element("uses-sdk", "targetSdkVersion")
+        return self.get_attribute_value("uses-sdk", "targetSdkVersion")
 
     def get_effective_target_sdk_version(self):
         """
@@ -1173,16 +1365,16 @@ class APK(object):
 
             :rtype: list
         """
-        return self.get_elements("uses-library", "name")
+        return list(self.get_all_attribute_value("uses-library", "name"))
 
     def get_features(self):
         """
         Return a list of all android:names found for the tag uses-feature
         in the AndroidManifest.xml
 
-        :return: list
+        :returns: list
         """
-        return self.get_elements("uses-feature", "name")
+        return list(self.get_all_attribute_value("uses-feature", "name"))
 
     def is_wearable(self):
         """
@@ -1193,7 +1385,7 @@ class APK(object):
         Not every app is setting this feature (not even the example Google provides),
         so it might be wise to not 100% rely on this feature.
 
-        :return: True if wearable, False otherwise
+        :returns: True if wearable, False otherwise
         """
         return 'android.hardware.type.watch' in self.get_features()
 
@@ -1202,7 +1394,7 @@ class APK(object):
         Checks if this application is build for TV (Leanback support)
         by checkin if it uses the feature 'android.software.leanback'
 
-        :return: True if leanback feature is used, false otherwise
+        :returns: True if leanback feature is used, false otherwise
         """
         return 'android.software.leanback' in self.get_features()
 
@@ -1212,16 +1404,16 @@ class APK(object):
         as this is the rule to get into the TV section of the Play Store
         See: https://developer.android.com/training/tv/start/start.html for more information.
 
-        :return: True if 'android.hardware.touchscreen' is not required, False otherwise
+        :returns: True if 'android.hardware.touchscreen' is not required, False otherwise
         """
-        return self.get_element('uses-feature', 'name', required="false", name="android.hardware.touchscreen") == "android.hardware.touchscreen"
+        return self.get_attribute_value('uses-feature', 'name', required="false", name="android.hardware.touchscreen") == "android.hardware.touchscreen"
 
     def get_certificate_der(self, filename):
         """
         Return the DER coded X.509 certificate from the signature file.
 
         :param filename: Signature filename in APK
-        :return: DER coded X.509 certificate as binary
+        :returns: DER coded X.509 certificate as binary
         """
         pkcs7message = self.get_file(filename)
 
@@ -1234,7 +1426,7 @@ class APK(object):
         Return a X.509 certificate object by giving the name in the apk file
 
         :param filename: filename of the signature file in the APK
-        :return: a :class:`Certificate` certificate
+        :returns: a :class:`Certificate` certificate
         """
         cert = self.get_certificate_der(filename)
         certificate = x509.Certificate.load(cert)
@@ -1418,24 +1610,27 @@ class APK(object):
                 # before the end of central dir...
 
                 # These things should not happen for APKs
-                assert this_disk == 0, "Not sure what to do with multi disk ZIP!"
-                assert disk_central == 0, "Not sure what to do with multi disk ZIP!"
+                if this_disk != 0:
+                    raise BrokenAPKError("Not sure what to do with multi disk ZIP!")
+                if disk_central != 0:
+                    raise BrokenAPKError("Not sure what to do with multi disk ZIP!")
                 break
             f.seek(-4, io.SEEK_CUR)
 
         if not offset_central:
             return
-        
+
         f.seek(offset_central)
         r, = unpack('<4s', f.read(4))
         f.seek(-4, io.SEEK_CUR)
-        assert r == self._PK_CENTRAL_DIR, "No Central Dir at specified offset"
+        if r != self._PK_CENTRAL_DIR:
+            raise BrokenAPKError("No Central Dir at specified offset")
 
         # Go back and check if we have a magic
         end_offset = f.tell()
         f.seek(-24, io.SEEK_CUR)
         size_of_block, magic = unpack('<Q16s', f.read(24))
-        
+
         self._is_signed_v2 = False
         self._is_signed_v3 = False
 
@@ -1445,7 +1640,8 @@ class APK(object):
         # go back size_of_blocks + 8 and read size_of_block again
         f.seek(-(size_of_block + 8), io.SEEK_CUR)
         size_of_block_start, = unpack("<Q", f.read(8))
-        assert size_of_block_start == size_of_block, "Sizes at beginning and and does not match!"
+        if size_of_block_start != size_of_block:
+            raise BrokenAPKError("Sizes at beginning and and does not match!")
 
         # Store all blocks
         while f.tell() < end_offset - 24:
@@ -1472,7 +1668,6 @@ class APK(object):
         if not self.is_signed_v3():
             return
 
-        certificates = []
         block_bytes = self._v2_blocks[self._APK_SIG_KEY_V3_SIGNATURE]
         block = io.BytesIO(block_bytes)
         view = block.getvalue()
@@ -1492,15 +1687,14 @@ class APK(object):
         #    * maxSDK
         #    * signatures
         #    * publickey
-        
         size_sequence = self.read_uint32_le(block)
-        assert size_sequence + 4 == len(block_bytes), "size of sequence and blocksize does not match"
-        
-        while block.tell() < len(block_bytes):
+        if size_sequence + 4 != len(block_bytes):
+            raise BrokenAPKError("size of sequence and blocksize does not match")
 
+        while block.tell() < len(block_bytes):
             off_signer = block.tell()
             size_signer = self.read_uint32_le(block)
-            
+
             # read whole signed data, since we might to parse
             # content within the signed data, and mess up offset
             len_signed_data = self.read_uint32_le(block)
@@ -1562,8 +1756,6 @@ class APK(object):
 
             self._v3_signing_data.append(signer)
 
-
-
     def parse_v2_signing_block(self):
         """
         Parse the V2 signing block and extract all features
@@ -1575,11 +1767,10 @@ class APK(object):
         if not self.is_signed_v2():
             return
 
-        certificates = []
         block_bytes = self._v2_blocks[self._APK_SIG_KEY_V2_SIGNATURE]
         block = io.BytesIO(block_bytes)
         view = block.getvalue()
-        
+
         # V2 signature Block data format:
         #
         # * signer:
@@ -1593,13 +1784,12 @@ class APK(object):
         #    * publickey
 
         size_sequence = self.read_uint32_le(block)
-        assert size_sequence + 4 == len(block_bytes), "size of sequence and blocksize does not match"
+        if size_sequence + 4 != len(block_bytes):
+            raise BrokenAPKError("size of sequence and blocksize does not match")
 
         while block.tell() < len(block_bytes):
-
             off_signer = block.tell()
             size_signer = self.read_uint32_le(block)
-            
 
             # read whole signed data, since we might to parse
             # content within the signed data, and mess up offset
@@ -1607,7 +1797,6 @@ class APK(object):
             signed_data_bytes = block.read(len_signed_data)
             signed_data = io.BytesIO(signed_data_bytes)
 
-                
             # Digests
             len_digests = self.read_uint32_le(signed_data)
             raw_digests = signed_data.read(len_digests)
@@ -1740,7 +1929,6 @@ class APK(object):
         Therefore this is just a list of all certificates found in all signers.
         """
         return [ x509.Certificate.load(cert) for cert in self.get_certificates_der_v2()]
-        
 
     def get_certificates_v1(self):
         """
@@ -1880,7 +2068,6 @@ class APK(object):
                 show_Certificate(c)
 
 
-
 def show_Certificate(cert, short=False):
     """
         Print Fingerprints, Issuer and Subject of an X509 Certificate.
@@ -1891,8 +2078,6 @@ def show_Certificate(cert, short=False):
         :type cert: :class:`asn1crypto.x509.Certificate`
         :type short: Boolean
     """
-
-
     print("SHA1 Fingerprint: {}".format(cert.sha1_fingerprint))
     print("SHA256 Fingerprint: {}".format(cert.sha256_fingerprint))
     print("Issuer: {}".format(get_certificate_name_string(cert.issuer.native, short=short)))
@@ -1911,6 +2096,7 @@ def ensure_final_value(packageName, arsc, value):
     if value:
         returnValue = value
         if value[0] == '@':
+            # TODO: @packagename:DEADBEEF is not supported here!
             try:  # can be a literal value or a resId
                 res_id = int('0x' + value[1:], 16)
                 res_id = arsc.get_id(packageName, res_id)[1]
@@ -1973,3 +2159,4 @@ def get_apkid(apkfile):
         versionName = ''  # versionName is expected to always be a str
 
     return appid, versionCode, versionName.strip('\0')
+
