@@ -15,14 +15,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from builtins import range
+from builtins import object
 import logging
 from collections import defaultdict
-from androguard.decompiler.dad.instruction import (Variable, ThisParam,
-                                                   Param)
+from androguard.decompiler.dad.instruction import (Variable, ThisParam, Param)
 from androguard.decompiler.dad.util import build_path, common_dom
 from androguard.decompiler.dad.node import Node
 
-
 logger = logging.getLogger('dad.control_flow')
 
 
@@ -47,11 +47,11 @@ class BasicReachDef(object):
                 if kill is not None:
                     self.defs[node][kill].add(i)
                     self.def_to_loc[kill].add(i)
-            for defs, values in self.defs[node].iteritems():
+            for defs, values in self.defs[node].items():
                 self.DB[node].add(max(values))
 
     def run(self):
-        nodes = self.g.rpo[:]
+        nodes = list(self.g.rpo)
         while nodes:
             node = nodes.pop(0)
             newR = set()
@@ -80,13 +80,13 @@ class BasicReachDef(object):
 
 
 def update_chain(graph, loc, du, ud):
-    '''
+    """
     Updates the DU chain of the instruction located at loc such that there is
     no more reference to it so that we can remove it.
     When an instruction is found to be dead (i.e it has no side effect, and the
     register defined is not used) we have to update the DU chain of all the
     variables that may me used by the dead instruction.
-    '''
+    """
     ins = graph.get_ins_from_loc(loc)
     for var in ins.get_used_vars():
         # We get the definition points of the current variable
@@ -116,14 +116,14 @@ def update_chain(graph, loc, du, ud):
 
 
 def dead_code_elimination(graph, du, ud):
-    '''
+    """
     Run a dead code elimination pass.
     Instructions are checked to be dead. If it is the case, we remove them and
     we update the DU & UD chains of its variables to check for further dead
     instructions.
-    '''
+    """
     for node in graph.rpo:
-        for i, ins in node.get_loc_with_ins()[:]:
+        for i, ins in node.get_loc_with_ins():
             reg = ins.get_lhs()
             if reg is not None:
                 # If the definition is not used, we check that the instruction
@@ -148,25 +148,25 @@ def dead_code_elimination(graph, du, ud):
 
 
 def clear_path_node(graph, reg, loc1, loc2):
-    for loc in xrange(loc1, loc2):
+    for loc in range(loc1, loc2):
         ins = graph.get_ins_from_loc(loc)
         logger.debug('  treat loc: %d, ins: %s', loc, ins)
         if ins is None:
             continue
         logger.debug('  LHS: %s, side_effect: %s', ins.get_lhs(),
-                                                   ins.has_side_effect())
+                     ins.has_side_effect())
         if ins.get_lhs() == reg or ins.has_side_effect():
             return False
     return True
 
 
 def clear_path(graph, reg, loc1, loc2):
-    '''
+    """
     Check that the path from loc1 to loc2 is clear.
     We have to check that there is no side effect between the two location
     points. We also have to check that the variable `reg` is not redefined
     along one of the possible pathes from loc1 to loc2.
-    '''
+    """
     logger.debug('clear_path: reg(%s), loc1(%s), loc2(%s)', reg, loc1, loc2)
     node1 = graph.get_node_from_loc(loc1)
     node2 = graph.get_node_from_loc(loc2)
@@ -189,7 +189,7 @@ def clear_path(graph, reg, loc1, loc2):
 
 
 def register_propagation(graph, du, ud):
-    '''
+    """
     Propagate the temporary registers between instructions and remove them if
     necessary.
     We process the nodes of the graph in reverse post order. For each
@@ -198,12 +198,12 @@ def register_propagation(graph, du, ud):
     its definition.
     We have to be careful to the side effects some instructions may have.
     To do the propagation, we use the computed DU and UD chains.
-    '''
+    """
     change = True
     while change:
         change = False
         for node in graph.rpo:
-            for i, ins in node.get_loc_with_ins()[:]:
+            for i, ins in node.get_loc_with_ins():
                 logger.debug('Treating instruction %d: %s', i, ins)
                 logger.debug('  Used vars: %s', ins.get_used_vars())
                 for var in ins.get_used_vars():
@@ -222,7 +222,7 @@ def register_propagation(graph, du, ud):
                     orig_ins = graph.get_ins_from_loc(loc)
                     logger.debug('     -> %s', orig_ins)
                     logger.debug('     -> DU(%s, %s) = %s', var, loc,
-                                                    du[var, loc])
+                                 du[var, loc])
 
                     # We defined some instructions as not propagable.
                     # Actually this is the case only for array creation
@@ -248,7 +248,7 @@ def register_propagation(graph, du, ud):
                         safe = True
                         orig_ins_used_vars = orig_ins.get_used_vars()
                         logger.debug('    variables used by the original '
-                                    'instruction: %s', orig_ins_used_vars)
+                                     'instruction: %s', orig_ins_used_vars)
                         for var2 in orig_ins_used_vars:
                             # loc is the location of the defined variable
                             # i is the location of the current instruction
@@ -293,7 +293,7 @@ def register_propagation(graph, du, ud):
                             continue
                         ud[var2, i].extend(old_ud)
                         logger.debug('\t  - ud(%s, %s) = %s', var2, i,
-                                                          ud[var2, i])
+                                     ud[var2, i])
                         ud.pop((var2, loc))
 
                         for def_loc in old_ud:
@@ -328,7 +328,8 @@ class DummyNode(Node):
 def group_variables(lvars, DU, UD):
     treated = defaultdict(list)
     variables = defaultdict(list)
-    for var, loc in sorted(DU):
+    # FIXME
+    for var, loc in sorted(DU, key=lambda x: (str(x[0]), str(x[1]))):
         if var not in lvars:
             continue
         if loc in treated[var]:
@@ -362,7 +363,7 @@ def split_variables(graph, lvars, DU, UD):
         nb_vars = max(lvars) + 1
     else:
         nb_vars = 0
-    for var, versions in variables.iteritems():
+    for var, versions in variables.items():
         nversions = len(versions)
         if nversions == 1:
             continue
@@ -408,7 +409,7 @@ def reach_def_analysis(graph, lparams):
         graph.add_edge(old_exit, new_exit)
         graph.rpo.append(new_exit)
 
-    analysis = BasicReachDef(graph, set(lparams))
+    analysis = BasicReachDef(graph, lparams)
     analysis.run()
 
     # The analysis is done, We can now remove the two special nodes.
@@ -420,10 +421,10 @@ def reach_def_analysis(graph, lparams):
 
 
 def build_def_use(graph, lparams):
-    '''
+    """
     Builds the Def-Use and Use-Def (DU/UD) chains of the variables of the
     method.
-    '''
+    """
     analysis = reach_def_analysis(graph, lparams)
 
     UD = defaultdict(list)
@@ -446,7 +447,7 @@ def build_def_use(graph, lparams):
                     UD[var, i].append(prior_def)
                 else:
                     intersect = analysis.def_to_loc[var].intersection(
-                                                            analysis.R[node])
+                        analysis.R[node])
                     UD[var, i].extend(intersect)
     DU = defaultdict(list)
     for var_loc, defs_loc in UD.items():
@@ -462,8 +463,8 @@ def place_declarations(graph, dvars, du, ud):
     for node in graph.post_order():
         for loc, ins in node.get_loc_with_ins():
             for var in ins.get_used_vars():
-                if (not isinstance(dvars[var], Variable)
-                    or isinstance(dvars[var], Param)):
+                if (not isinstance(dvars[var], Variable) or
+                        isinstance(dvars[var], Param)):
                     continue
                 var_defs_locs = ud[var, loc]
                 def_nodes = set()
@@ -478,7 +479,7 @@ def place_declarations(graph, dvars, du, ud):
                 common_dominator = def_nodes.pop()
                 for def_node in def_nodes:
                     common_dominator = common_dom(
-                                      idom, common_dominator, def_node)
+                        idom, common_dominator, def_node)
                 if any(var in range(*common_dominator.ins_range)
                        for var in ud[var, loc]):
                     continue
