@@ -15,23 +15,27 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from builtins import str
+from builtins import zip
+from builtins import range
+from past.builtins import basestring
+from builtins import object
 import logging
 from struct import unpack
 from androguard.decompiler.dad.util import get_type
 from androguard.decompiler.dad.opcode_ins import Op
-from androguard.decompiler.dad.instruction import (Constant, ThisParam,
-                                                   BinaryExpression,
-                                                   BaseClass,
-                                                   InstanceExpression,
-                                                   NewInstance,
-                                                   Variable,
-                                                   BinaryCompExpression)
-
+from androguard.decompiler.dad.instruction import (
+    Constant, ThisParam, BinaryExpression, BaseClass, InstanceExpression,
+    NewInstance, Variable, BinaryCompExpression)
 
 logger = logging.getLogger('dad.writer')
 
 
 class Writer(object):
+    """
+    Transforms a method into Java code.
+
+    """
     def __init__(self, graph, method):
         self.graph = graph
         self.method = method
@@ -103,10 +107,16 @@ class Writer(object):
             rhs.visit(self)
         self.end_ins()
 
-    #TODO: prefer this class as write_ind_visit_end that should be deprecated
+    # TODO: prefer this class as write_ind_visit_end that should be deprecated
     # at the end
-    def write_ind_visit_end_ext(self, lhs, before, s, after, rhs=None,
-                                data=None, subsection='UNKNOWN_SUBSECTION'):
+    def write_ind_visit_end_ext(self,
+                                lhs,
+                                before,
+                                s,
+                                after,
+                                rhs=None,
+                                data=None,
+                                subsection='UNKNOWN_SUBSECTION'):
         self.write_ind()
         lhs.visit(self)
         self.write(before + s + after)
@@ -120,11 +130,14 @@ class Writer(object):
     def write_inplace_if_possible(self, lhs, rhs):
         if isinstance(rhs, BinaryExpression) and lhs == rhs.var_map[rhs.arg1]:
             exp_rhs = rhs.var_map[rhs.arg2]
-            if rhs.op in '+-' and isinstance(exp_rhs, Constant) and\
-                                  exp_rhs.get_int_value() == 1:
+            if rhs.op in '+-' and isinstance(exp_rhs, Constant) and \
+                            exp_rhs.get_int_value() == 1:
                 return self.write_ind_visit_end(lhs, rhs.op * 2, data=rhs)
             return self.write_ind_visit_end(
-                lhs, ' %s= ' % rhs.op, exp_rhs, data=rhs)
+                lhs,
+                ' %s= ' % rhs.op,
+                exp_rhs,
+                data=rhs)
         return self.write_ind_visit_end(lhs, ' = ', rhs, data=rhs)
 
     def visit_ins(self, ins):
@@ -149,22 +162,20 @@ class Writer(object):
             self.write(name)
             self.write_ext(('NAME_METHOD_PROTOTYPE', '%s' % name, self.method))
         else:
-            self.write(
-                '%s %s' % (get_type(self.method.type), self.method.name))
+            self.write('%s %s' % (get_type(self.method.type), self.method.name))
             self.write_ext(
                 ('PROTOTYPE_TYPE', '%s' % get_type(self.method.type)))
             self.write_ext(('SPACE', ' '))
             self.write_ext(
-                ('NAME_METHOD_PROTOTYPE',
-                '%s' % self.method.name, self.method))
+                ('NAME_METHOD_PROTOTYPE', '%s' % self.method.name, self.method))
         params = self.method.lparams
         if 'static' not in access:
             params = params[1:]
         proto = ''
         self.write_ext(('PARENTHESIS_START', '('))
         if self.method.params_type:
-            proto = ', '.join(['%s p%s' % (get_type(p_type), param) for
-                        p_type, param in zip(self.method.params_type, params)])
+            proto = ', '.join(['%s p%s' % (get_type(p_type), param) for p_type,
+                                                                        param in zip(self.method.params_type, params)])
             first = True
             for p_type, param in zip(self.method.params_type, params):
                 if not first:
@@ -173,8 +184,7 @@ class Writer(object):
                     first = False
                 self.write_ext(('ARG_TYPE', '%s' % get_type(p_type)))
                 self.write_ext(('SPACE', ' '))
-                self.write_ext(
-                    ('NAME_ARG', 'p%s' % param, p_type, self.method))
+                self.write_ext(('NAME_ARG', 'p%s' % param, p_type, self.method))
         self.write_ext(('PARENTHESIS_END', ')'))
         self.write('(%s)' % proto)
         if self.graph is None:
@@ -281,9 +291,9 @@ class Writer(object):
             self.write('%s}\n' % self.space(), data="IF_END_2")
             self.visit_node(cond.false)
         elif follow is not None:
-            if cond.true in (follow, self.next_case) or\
-                                                cond.num > cond.true.num:
-                             # or cond.true.num > cond.false.num:
+            if cond.true in (follow, self.next_case) or \
+                            cond.num > cond.true.num:
+                # or cond.true.num > cond.false.num:
                 cond.neg()
                 cond.true, cond.false = cond.false, cond.true
             self.if_follow.append(follow)
@@ -343,7 +353,8 @@ class Writer(object):
             self.inc_ind()
             for case in switch.node_to_case[node]:
                 self.write(
-                    '%scase %d:\n' % (self.space(), case), data="CASE_XX")
+                    '%scase %d:\n' % (self.space(), case),
+                    data="CASE_XX")
             if i + 1 < len(cases):
                 self.next_case = cases[i + 1]
             else:
@@ -414,14 +425,16 @@ class Writer(object):
         if not var.declared:
             var_type = var.get_type() or 'unknownType'
             self.write('%s%s v%s' % (
-                self.space(), get_type(var_type),
-                var.name), data="DECLARATION")
+                self.space(), get_type(var_type), var.name),
+                       data="DECLARATION")
             self.end_ins()
 
     def visit_constant(self, cst):
+        # FIXME: basestring should be replaced
         if isinstance(cst, basestring):
             return self.write(string(cst), data="CONSTANT_STRING")
-        self.write('%r' % cst, data="CONSTANT_INTEGER")  # INTEGER or also others?
+        self.write('%r' % cst,
+                   data="CONSTANT_INTEGER")  # INTEGER or also others?
 
     def visit_base_class(self, cls, data=None):
         self.write(cls)
@@ -445,6 +458,9 @@ class Writer(object):
     def visit_this(self):
         self.write('this', data="THIS")
 
+    def visit_super(self):
+        self.write('super')
+
     def visit_assign(self, lhs, rhs):
         if lhs is not None:
             return self.write_inplace_if_possible(lhs, rhs)
@@ -477,8 +493,13 @@ class Writer(object):
 
     def visit_put_instance(self, lhs, name, rhs, data=None):
         self.write_ind_visit_end_ext(
-            lhs, '.', '%s' % name, ' = ', rhs,
-            data=data, subsection='NAME_CLASS_ASSIGNMENT')
+            lhs,
+            '.',
+            '%s' % name,
+            ' = ',
+            rhs,
+            data=data,
+            subsection='NAME_CLASS_ASSIGNMENT')
 
     def visit_new(self, atype, data=None):
         self.write('new %s' % get_type(atype))
@@ -486,11 +507,14 @@ class Writer(object):
         self.write_ext(
             ('NAME_CLASS_NEW', '%s' % get_type(atype), data.type, data))
 
-    def visit_invoke(self, name, base, ptype, rtype, args, invokeInstr=None):
+    def visit_invoke(self, name, base, ptype, rtype, args, invokeInstr):
         if isinstance(base, ThisParam):
-            if name == '<init>' and self.constructor and len(args) == 0:
-                self.skip = True
-                return
+            if name == '<init>':
+                if self.constructor and len(args) == 0:
+                    self.skip = True
+                    return
+                if invokeInstr and base.type[1:-1].replace('/', '.') != invokeInstr.cls:
+                    base.super = True
         base.visit(self)
         if name != '<init>':
             if isinstance(base, BaseClass):
@@ -505,7 +529,7 @@ class Writer(object):
                         call_name = "%s -> %s" % (base2base.type, name)
                         break
                     elif (hasattr(base2base, "base") and
-                          hasattr(base2base, "var_map")):
+                              hasattr(base2base, "var_map")):
                         continue
                     else:
                         call_name = "UNKNOWN_TODO"
@@ -519,8 +543,8 @@ class Writer(object):
             self.write('.%s' % name)
             self.write_ext(('INVOKE', '.'))
             self.write_ext(
-                ('NAME_METHOD_INVOKE',
-                 '%s' % name, call_name, ptype, rtype, base, invokeInstr))
+                ('NAME_METHOD_INVOKE', '%s' % name, call_name, ptype, rtype,
+                 base, invokeInstr))
         self.write('(', data="PARAM_START")
         comma = False
         for arg in args:
@@ -582,12 +606,20 @@ class Writer(object):
         data = value.get_data()
         tab = []
         elem_size = value.element_width
-        if elem_size == 4:
-            for i in range(0, value.size * 4, 4):
-                tab.append('%s' % unpack('i', data[i:i + 4])[0])
-        else:  # FIXME: other cases
-            for i in range(value.size):
-                tab.append('%s' % unpack('b', data[i])[0])
+
+        # Set type depending on size of elements
+        data_types = {1: 'b', 2: 'h', 4: 'i', 8: 'd'}
+
+        if elem_size in data_types:
+            elem_id = data_types[elem_size]
+        else:
+            # FIXME for other types we just assume bytes...
+            logger.warning("Unknown element size {} for array. Assume bytes.".format(elem_size))
+            elem_id = 'b'
+            elem_size = 1
+
+        for i in range(0, value.size*elem_size, elem_size):
+            tab.append('%s' % unpack(elem_id, data[i:i+elem_size])[0])
         self.write(', '.join(tab), data="COMMA")
         self.write('}', data="ARRAY_FILLED_END")
         self.end_ins()
@@ -596,8 +628,7 @@ class Writer(object):
         var.declared = True
         var_type = var.get_type() or 'unknownType'
         self.write('%s v%s' % (get_type(var_type), var.name))
-        self.write_ext(
-            ('EXCEPTION_TYPE', '%s' % get_type(var_type), data.type))
+        self.write_ext(('EXCEPTION_TYPE', '%s' % get_type(var_type), data.type))
         self.write_ext(('SPACE', ' '))
         self.write_ext(
             ('NAME_CLASS_EXCEPTION', 'v%s' % var.value(), data.type, data))
@@ -669,16 +700,22 @@ class Writer(object):
 
 
 def string(s):
+    """
+    Convert a string to a escaped ASCII representation including quotation marks
+    :param s: a string
+    :return: ASCII escaped string
+    """
     ret = ['"']
-    for c in s.decode('utf8'):
-        if c >= ' ' and c < '\x7f':
+    for c in s:
+        if ' ' <= c < '\x7f':
             if c == "'" or c == '"' or c == '\\':
                 ret.append('\\')
             ret.append(c)
             continue
         elif c <= '\x7f':
             if c in ('\r', '\n', '\t'):
-                ret.append(c.encode('unicode-escape'))
+                # unicode-escape produces bytes
+                ret.append(c.encode('unicode-escape').decode("ascii"))
                 continue
         i = ord(c)
         ret.append('\\u')
@@ -687,4 +724,4 @@ def string(s):
         ret.append('%x' % ((i >> 4) & 0x0f))
         ret.append('%x' % (i & 0x0f))
     ret.append('"')
-    return ''.join(ret).encode('utf8')
+    return ''.join(ret)
