@@ -1,14 +1,15 @@
-from __future__ import absolute_import
 from __future__ import division
-from __future__ import print_function
 
 import os
 import mmap
 
-from .BinViewMode import *
-from .HexViewMode import *
-from .DisasmViewMode import *
-from .SourceViewMode import *
+from androguard.gui.BinViewMode import BinViewMode
+from androguard.gui.HexViewMode import HexViewMode
+from androguard.gui.DisasmViewMode import DisasmViewMode
+from androguard.gui.SourceViewMode import SourceViewMode
+
+import logging
+log = logging.getLogger("androguard.gui")
 
 
 class Observer(object):
@@ -30,7 +31,7 @@ class DataModel(Observer):
 
     @dataOffset.setter
     def dataOffset(self, value):
-        print("DATA OFFSET", value)
+        log.debug("DATA OFFSET %s", value)
         self._lastOffset = self._dataOffset
         self._dataOffset = value
 
@@ -220,6 +221,7 @@ class FileDataModel(DataModel):
         try:
             self._f = open(self._filename, "r+b")
         except OSError as e:
+            log.exception("File could not be opened for writing: %s", e)
             # could not open for writing
             return False
         self._f.write(self._mapped)
@@ -306,7 +308,6 @@ class BufferDataModel(DataModel):
 
 class ApkModel(DataModel):
     def __init__(self, apkobj):
-        print(apkobj)
         self._filename = str(apkobj)
         self.raw = apkobj.get_raw()
         self.data = MyByte(self.raw)
@@ -331,9 +332,15 @@ class ApkModel(DataModel):
 
 
 class DexClassModel(DataModel):
-    def __init__(self, current_class):
+    def __init__(self, current_class, dx):
+        """
+
+        :param current_class: a ClassDefItem
+        :param dx: a Analysis object
+        """
         self.current_class = current_class
         self._filename = current_class.get_name()
+        self.dx = dx
 
         raw = self.GetRawData(current_class)
         super(DexClassModel, self).__init__(raw)
