@@ -1,15 +1,13 @@
-from __future__ import absolute_import
 from __future__ import division
-from __future__ import print_function
 
 from PyQt5 import QtGui, QtCore
 from builtins import hex
 from builtins import range
 
 from androguard.core.bytecodes import dvm
-from . import TextSelection
-from .ViewMode import *
-from .cemu import *
+from androguard.gui import TextSelection
+from androguard.gui.ViewMode import ViewMode
+from androguard.gui.cemu import ConsoleEmulator, Directions
 
 import logging
 log = logging.getLogger("androguard.gui")
@@ -198,10 +196,10 @@ class DisasmViewMode(ViewMode):
 
         self.OPCODES = []
 
-        vm_analysis = self.dataModel.current_class.CM.get_vmanalysis()
+        vm_analysis = self.dataModel.dx
 
         methods = [i for i in self.dataModel.current_class.get_methods()]
-        print(methods)
+        log.debug(methods)
         methods = sorted(methods, key=lambda x: x.get_address(), reverse=True)
 
         offset = 0
@@ -220,7 +218,7 @@ class DisasmViewMode(ViewMode):
 
         self.max_offset = offset
 
-        print(sorted(self.CACHE_IDX_OPCODES_OFF.keys()))
+        log.debug(sorted(self.CACHE_IDX_OPCODES_OFF.keys()))
 
         self.textPen = QtGui.QPen(self.themes['pen'], 0, QtCore.Qt.SolidLine)
         self.resize(width, height)
@@ -239,7 +237,7 @@ class DisasmViewMode(ViewMode):
         return length
 
     def FeedOpcodes(self, cnt):
-        print('FEED ', cnt)
+        log.debug('FEED %s', cnt)
         self.OPCODES = []
         for i in range(0, min(cnt, len(self.CACHE_OPCODES))):
             ins = self.CACHE_OPCODES[i]
@@ -325,7 +323,7 @@ class DisasmViewMode(ViewMode):
     def drawCursor(self, qp):
         cursorX, cursorY = self.cursor.getPosition()
 
-        print(cursorX, cursorY)
+        log.debug("%s / %s", cursorX, cursorY)
 
         xstart = cursorX
 
@@ -468,20 +466,14 @@ class DisasmViewMode(ViewMode):
         if self.dataModel.getOffset() in self.Paints:
             self.refresh = False
             self.qpix = QtGui.QPixmap(self.Paints[self.dataModel.getOffset()])
-            # print 'hit'
             self.drawAdditionals()
             return
 
         if self.refresh or refresh:
             qp = QtGui.QPainter()
             qp.begin(self.qpix)
-            # viewport
-            # qp.fillRect(0, 0, self.COLUMNS * self.fontWidth,  self.ROWS * self.fontHeight, self.backgroundBrush)
 
-            # start = time()
             self.drawTextMode(qp)
-            # end = time() - start
-            # print 'Time ' + str(end)
             self.refresh = False
             qp.end()
 
@@ -528,7 +520,7 @@ class DisasmViewMode(ViewMode):
         return 0
 
     def _drawRow(self, qp, cemu, row, asm, offset=-1):
-        print('DRAW AN INSTRUCTION', asm, row, asm.get_name(), len(asm.get_operands(offset)), hex(self.getPageOffset()))
+        log.debug('DRAW AN INSTRUCTION %s %s %s %s %s', asm, row, asm.get_name(), len(asm.get_operands(offset)), hex(self.getPageOffset()))
 
         qp.setPen(QtGui.QPen(QtGui.QColor(192, 192, 192), 1, QtCore.Qt.SolidLine))
 
@@ -642,7 +634,7 @@ class DisasmViewMode(ViewMode):
             cemu.write(s[idx:])
 
     def drawTextMode(self, qp):
-        print('OFFSET', self.dataModel.getOffset())
+        log.debug('OFFSET %s', self.dataModel.getOffset())
         # draw background
         qp.fillRect(0, 0, self.COLUMNS * self.fontWidth, self.ROWS * self.fontHeight, self.backgroundBrush)
 
@@ -682,7 +674,7 @@ class DisasmViewMode(ViewMode):
         return None
 
     def goTo(self, offset):
-        print("GOTO ", offset)
+        log.debug("GOTO %s", offset)
 
         tsize = sum([opcode.get_length() for opcode in self.OPCODES])
 
@@ -713,7 +705,7 @@ class DisasmViewMode(ViewMode):
         self.scroll(0, -number * self.ROWS, cachePix=cachePix, pageOffset=pageOffset)
 
     def scroll_v(self, dy, cachePix=None, pageOffset=None):
-        print('scroll_v', dy, cachePix, pageOffset, hex(self.getCursorAbsolutePosition()))
+        log.debug('scroll_v %s %s %s %s', dy, cachePix, pageOffset, hex(self.getCursorAbsolutePosition()))
 
         RowsToDraw = []
         factor = abs(dy)
@@ -724,14 +716,14 @@ class DisasmViewMode(ViewMode):
                 tsize = sum([asm.get_length() for asm in self.OPCODES])
                 current_offset = self.dataModel.getOffset() + tsize
                 if current_offset not in self.CACHE_IDX_OPCODES_OFF:
-                    print('INVALID OFFSET', hex(current_offset))
+                    log.debug('INVALID OFFSET %s', hex(current_offset))
                     return
 
                 current_idx = self.CACHE_IDX_OPCODES_OFF[current_offset] - 1
-                print("IDX", current_idx, hex(current_offset))
+                log.debug("IDX %s %s", current_idx, hex(current_offset))
 
                 if current_idx + 1 >= len(self.CACHE_OPCODES):
-                    print('END OF DATA')
+                    log.debug('END OF DATA')
                     return
 
                 current_idx += 1
@@ -739,7 +731,7 @@ class DisasmViewMode(ViewMode):
             if dy >= 0:
                 current_offset = self.dataModel.getOffset()
                 current_idx = self.CACHE_IDX_OPCODES_OFF[current_offset]
-                print("IDX", current_idx, hex(current_offset))
+                log.debug("IDX %s %s", current_idx, hex(current_offset))
                 # start = self.CACHE_OPCODES[self.CACHE_IDX_OPCODES_OFF[self.getCursorAbsolutePosition()]-1]
                 current_idx -= 1
 
@@ -765,7 +757,7 @@ class DisasmViewMode(ViewMode):
             if dy > 0:
                 RowsToDraw.append((-row - 1, newins))
 
-        print('ROW TO DRAW', RowsToDraw)
+        log.debug('ROW TO DRAW %s', RowsToDraw)
         if len(RowsToDraw) < abs(dy):
             # maybe we couldn't draw dy rows (possible we reached the beginning of the data to early), recalculate dy
             dy = len(RowsToDraw) * dy / abs(dy)
@@ -798,10 +790,9 @@ class DisasmViewMode(ViewMode):
             self._drawRow(qp, cemu, dy + row, asm)
 
         qp.end()
-        print('')
 
     def scroll(self, dx, dy, cachePix=None, pageOffset=None):
-        print('scroll ', dx, dy, self.dataModel.inLimits((self.dataModel.getOffset() - dx)), 'offset',
+        log.debug('scroll %s %s %s %s %s', dx, dy, self.dataModel.inLimits((self.dataModel.getOffset() - dx)), 'offset',
               self.dataModel.getOffset())
         if dx != 0:
             if self.dataModel.inLimits((self.dataModel.getOffset() - dx)):
@@ -812,14 +803,14 @@ class DisasmViewMode(ViewMode):
         if dy != 0:
             if dy > 0:
                 if self.dataModel.getOffset() == 0:
-                    print('OFFSET == 0')
+                    log.debug('OFFSET == 0')
                     return
 
             if dy < 0:
                 tsize = sum([asm.get_length() for asm in self.OPCODES])
 
                 if self.dataModel.getOffset() + tsize == self.dataModel.getDataSize():
-                    print('END')
+                    log.debug('END')
                     return
 
             self.scroll_v(dy, cachePix, pageOffset)
